#include "BasicBlockWidget.hxx"
#include "gui/Mainwindow.hxx"

BasicBlockWidget::BasicBlockWidget(const QString& name, BasicBlock * block,
                                   Mainwindow * mainwindow)
	: width(270), height(45)
	, _proxy(this), name(name)
	, block(block), mainwindow(mainwindow) {
	next[0] = NULL;	next[1] = NULL;
	_widget.setStyleSheet("QWidget { background-color : #ddddff; }");
	_widget.setLayout(&_layout);
	_layout.setContentsMargins(0, 0, 0, 0);
	_proxy.setWidget(&_widget);
	_proxy.setPos(5, 20);

	if (width < 250) width = 250;
}

void BasicBlockWidget::addItem(uint8_t* bytes, size_t num_bytes,
                               QString line, const QString& href) {
	QString bytestring;

	for (size_t i(0); i < num_bytes; ++i) {
		const char * hexdigits = "0123456789ABCDEF";
		bytestring += hexdigits[(bytes[i] >> 4) & 0xF];
		bytestring += hexdigits[bytes[i] & 0xF];
		bytestring += ' ';
	}

	int current_row = _layout.rowCount();
	QLabel * bytestring_label = new QLabel(bytestring);
	bytestring_label->setTextInteractionFlags(Qt::TextSelectableByMouse|
	                                          Qt::LinksAccessibleByMouse);
	bytestring_label->setWordWrap(false);
	_layout.addWidget(bytestring_label, current_row, 0);
	bytestring_label->setVisible(true);

	line = line.replace('\t', ' ').toHtmlEscaped();
	if (href != "") {
		line = "<a href=\"" + href + "\">" + line + "</a>";
	}
	QLabel * instruction_label = new QLabel(line);
	instruction_label->setTextInteractionFlags(Qt::TextSelectableByMouse|
	                                           Qt::LinksAccessibleByMouse);
	instruction_label->setWordWrap(false);
	instruction_label->setFocusPolicy(Qt::StrongFocus);
	if (href != "") {
		QObject::connect(instruction_label, &QLabel::linkActivated,
		                 [=](QString str) {
			                 if (str.startsWith("function:")) {
					                 QString address = str.remove("function:");
					                 mainwindow->switchMainPlaneToAddress(address.toInt(NULL, 16));
				                 }
		                 });
	}
	_layout.addWidget(instruction_label, current_row, 1);
	instruction_label->setVisible(true);

	width = 12 + _widget.childrenRect().width();
	height = 25 + _widget.childrenRect().height();

	if (width < 250) width = 250;
	_widget.resize(width - 12, height - 25);
}
